<?php

namespace Drupal\page_notifications\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\examples\Utility\DescriptionTemplateTrait;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\page_notifications\Form\AccessVerificationStep;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;

/**
 * Controller routines for page example routes.
 */
class SubscriberPage extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  protected function getModuleName() {
    return 'subscriberpage';
  }

  public function subscriberpage($user_token = NULL) {
    if ($user_token && !is_null($user_token)) {
      $header = array(
        // We make it sortable by name.
        array('data' => $this->t('Watching pages list'), 'field' => 'title', 'sort' => 'asc'),
        array('data' => $this->t('')),
      );

      $query = \Drupal::entityQuery('node')
        ->condition('type', 'page_notify_subscriptions')
        ->condition('field_page_notify_token_user_id', $user_token, '=')
        ->condition('status', 1)
        ->sort('created', 'DESC')
        ->pager(10);
      $records = $query->execute();

      // Populate the rows.
      $rows = array();
      foreach ($records as $record) {
        $node_record = \Drupal\node\Entity\Node::load($record);
        $field_token_notify = $node_record->get("field_page_notify_token")->getValue();
        $field_node_id_notify = $node_record->get("field_page_notify_node_id")->getValue();

        $subscriptions_record = \Drupal\node\Entity\Node::load($field_node_id_notify[0]['value']);
          $rows[] = array('data' => array(
            'title' => new FormattableMarkup('<a href="@page_url">@page_title</a>',
              [
                '@page_title' => $subscriptions_record->getTitle(),
                '@page_url' => $subscriptions_record->toUrl()->toString(),
              ]),
            'cancel_one' => new FormattableMarkup('<a id="notify-cancel-@token" href="/nojs/cancel_subscription/@token" class="use-ajax btn btn-default notify-cancel-subscription">@name</a>',
                ['@name' => 'Stop Watching', '@token' => $field_token_notify[0]['value']]
              ),
          ));

        if ($rows) {
          $cancelall =  '<a id="notify-cancel-all" href="/nojs/cancel_all/' . $user_token .'" class="use-ajax btn btn-default notify-cancel-all-subscription">Unsubscribe from all</a>';
          $header = array(
            // We make it sortable by name.
            array('data' => $this->t('Page Name'), 'field' => 'title', 'sort' => 'asc'),
            array('data' => $this->t($cancelall)),
          );
        }
        else {
          $build = array();
        }
      }

      $page_name = '<h1>Manage Your Page Watching Subscriptions</h1>';
      $build['page_name'] = [
        '#markup' => $page_name,
        '#attributes' => [
          'class' => ['page-notifications-user-list-page-name'],
        ],
      ];
      $build['config_table'] = array(
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => t('No records found'),
        '#attributes' => [
          'class' => ['page-notifications-block-subscriberpage'],
          'id' => 'page-notifications-block-subscriberpage',
          'no_striping' => TRUE,
        ],
      );
      $build['pager'] = array(
        '#type' => 'pager'
      );
      return $build;
    }
  }

  public function cancel_subscription($token) {
      $response = new AjaxResponse();
      page_notifications_delete_record($token);
      $response->addCommand(new ReplaceCommand('#notify-cancel-' . $token, '<span class="notify-cancel-cancelled">Cancelled</span>'));
      return $response;
  }

  public function cancel_all($user_token) {
      $response = new AjaxResponse();
      page_notifications_delete_all_records($user_token);
      $response->addCommand(new ReplaceCommand('#notify-cancel-all', '<span class="notify-cancel-all-cancelled">All cancelled</span>'));
      $response->addCommand(new ReplaceCommand('#notify-cancel-', "Cancelled"));
      return $response;
  }
}



function checkForRecord($subscription_token_url, $email) {
  $record = current(\Drupal::entityTypeManager()->getStorage('node')
    ->loadByProperties([
      'field_page_notify_token' => $subscription_token_url,
      'field_page_notify_email' => $email
    ])
  );
  if ($record) {
    return $record;
  }
  else {
    return FALSE;
  }
}

function getAllRecords($email) {
  $query = \Drupal::entityQuery('node')
    ->condition('status', 1)
    ->condition('field_page_notify_email', $email, '=');
  $records = $query->execute();
  foreach ($records as $key => $record) {
    $node = \Drupal\node\Entity\Node::load($record);
    $nodes[] = $node;
  }
  if ($nodes) {
    return $nodes;
  }
  else {
    return FALSE;
  }
}

function page_notifications_delete_record($token) {
  $num_deleted = \Drupal::entityQuery("node")
    ->condition("type", "page_notify_subscriptions")
    ->condition("field_page_notify_token", $token)
    ->accessCheck(FALSE)
    ->execute();
  $storage_handler = \Drupal::entityTypeManager()->getStorage("node");
  $entities = $storage_handler->loadMultiple($num_deleted);
  $storage_handler->delete($entities);
}

function page_notifications_delete_all_records($user_token) {
  //TODO possibly add condition to check for user token
  $num_deleted = \Drupal::entityQuery("node")
    ->condition("type", "page_notify_subscriptions")
    ->condition("field_page_notify_token_user_id", $user_token)
    ->accessCheck(FALSE)
    ->execute();
  $storage_handler = \Drupal::entityTypeManager()->getStorage("node");
  $entities = $storage_handler->loadMultiple($num_deleted);
  $storage_handler->delete($entities);
}
