<?php

namespace Drupal\page_notifications\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Mail\MailManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Component\Utility\EmailValidator;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\node\Entity\Node;

/**
 * File test form class.
 *
 * @ingroup page_notifications
 */
class EmailConfirmationPage extends FormBase {

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidator
   */
  protected $emailValidator;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new EmailUnsubscribePage.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Component\Utility\EmailValidator $email_validator
   *   The email validator.
   */
  public function __construct(MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager, EmailValidator $email_validator) {
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
    $this->emailValidator = $email_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $form = new static(
      $container->get('plugin.manager.mail'),
      $container->get('language_manager'),
      $container->get('email.validator')
    );
    $form->setMessenger($container->get('messenger'));
    $form->setStringTranslation($container->get('string_translation'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'page_notifications_confirmation';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $email = NULL, $subscription_token = NULL) {

    if($email && !is_null($email) && $subscription_token && !is_null($subscription_token)) {
      $subscription_token_pieces = explode("-", $subscription_token);
      $node_id = $subscription_token_pieces[0];
      $subscription_token_raw = $subscription_token_pieces[1];
      $host = \Drupal::request()->getSchemeAndHttpHost();
      $inrecords = secondCheckIfRecordExistNode($email, $node_id);
      if ($inrecords && $inrecords->isPublished() == true) {
        $unsubscribe_link = $host . "/page-notifications/verify-list/" . $subscription_token;
        $form['intro'] = [
          '#markup' => $this->t('<div clas="page-notifications-find-my-subscriptions"> Find your subscriptions <a href='. $unsubscribe_link .'>here</a>.</div>'),
        ];
        return $form;
      }
      else {
        $node = \Drupal\node\Entity\Node::load($node_id);
        $node_title = $node->getTitle();
        $subscribed_node_url = $node->toUrl()->setAbsolute()->toString();
        $user_token_exist = \Drupal::service('load.databaseinnfo.service')->pageNotifyGetUserToken($email);
        if ($user_token_exist && $user_token_exist != false) {
          $user_token = $user_token_exist['field_page_notify_token_user_id'];
        }
        else {
          $user_token = \Drupal::service('load.databaseinnfo.service')->page_notifications_generateRandom_user_token();
        }

        // Create new subscription
        $new_submition = Node::create([
          'type' => 'page_notify_subscriptions',
          'title' => 'Subscription to - ' . $node_id . ' - ' . $subscription_token_raw,
          'field_page_notify_node_id' => $node_id,
          'field_page_notify_email' => $email,
          'field_page_notify_token' => $subscription_token_raw,
          'field_page_notify_token_user_id' => $user_token,
        ]);
        $new_submition->save();

        //TODO make this check on node or path
        $all_subscriptions_url = $host . '/page-notifications/verify-list/' . $node_id . '-' . $subscription_token_raw;
        $unsubscribe_link = $host . "/page-notifications/unsubscribe/" . $subscription_token;
        // Load template for confirmation email
        $template_info = \Drupal::service('load.databaseinnfo.service')->get_notify_email_template();
        // Set the FROM email
        if ($template_info['from_email']) {
          $from = $template_info['from_email'];
        }
        else {
          $from = \Drupal::config('system.site')->get('mail');
        }

        $subject_replacements = array(
          '[notify_user_name]' => '',
          '[notify_user_email]' => $email,
          '[notify_verify_url]' => '',
          '[notify_subscribe_url]' => '',
          '[notify_unsubscribe_url]' => '',
          '[notify_user_subscribtions]' => '',
          '[notify_node_title]' => $node_title,
          '[notify_node_url]' => '',
          '[notify_notes]' => '',
        );
        $body_replacements = array(
          '[notify_user_name]' => '',
          '[notify_user_email]' => $email,
          '[notify_verify_url]' => '',
          '[notify_subscribe_url]' => '',
          '[notify_unsubscribe_url]' => $unsubscribe_link,
          '[notify_user_subscribtions]' => $all_subscriptions_url,
          '[notify_node_title]' => $node_title,
          '[notify_node_url]' => $subscribed_node_url,
          '[notify_notes]' => '',
        );

        $tokanized_subject = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['confirmation_email_subject'], $subject_replacements);
        $tokanized_body = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['confirmation_email_text'], $body_replacements);
        strval($tokanized_subject);


        /*$message['headers'] = array(
          'content-type' => 'text/html',
          'MIME-Version' => '1.0',
          'reply-to' => $from,
          'from' => $from
        );
        $message['to'] = $email;
        $message['subject'] = $tokanized_subject;
        $message['body'] = $tokanized_body;
        $send_confirmation_mail->mail($message);*/

        $message['to'] = $email;
        $message['subject'] = $tokanized_subject;
        $message['body'] = $tokanized_body;
        $result = \Drupal::service('plugin.manager.mail')->mail(
           'page_notifications',
           'configuration_email',
           $email,
           \Drupal::languageManager()->getDefaultLanguage()->getId(),
           $message
         );

        $tokanized_notify_confirmation_web_page_message = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['confirmation_web_page_message'], $body_replacements);
        if ($template_info['confirmation_web_page_message']) {
          $form['intro'] = [
            '#type' => 'processed_text',
            '#text' => $this->t($tokanized_notify_confirmation_web_page_message),
            '#format' => 'full_html',
          ];
        }
        else {
          $form['intro'] = [
            '#markup' => new FormattableMarkup('<br /><p>You are now subscribed to</p> <h2><a href="@link">@title</a></h2><br />
              <p><a href="@all_subscriptions_url">Manage Your Page Watching Subscriptions</a>.</p>',
              ['@title' => $node_title, '@link' => $subscribed_node_url, '@all_subscriptions_url' => $all_subscriptions_url]
            ),
          ];
        }
        return $form;
      }
    }
    else {
      \Drupal::messenger()->addStatus(t('Sorry, no data on this.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
  }
}

function secondCheckIfRecordExistNode($email, $node) {
  $record = current(\Drupal::entityTypeManager()->getStorage('node')
    ->loadByProperties([
      'field_page_notify_email' => $email,
      'field_page_notify_node_id' => $node
    ])
  );
  if ($record) {
    return $record;
  }
  else {
    return FALSE;
  }
}
