<?php

namespace Drupal\page_notifications\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Component\Utility\EmailValidator;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use \Drupal\node\Entity\Node;
use \Drupal\Component\Utility\UrlHelper;
use Drupal\filter\Element\ProcessedText;
use Drupal\Core\Render\BubbleableMetadata;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Routing\RouteMatchInterface;


use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CssCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
/**
 * Submit a form without a page reload.
 */
class PageNotificationsBlockForm extends FormBase {
  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidator
   */
  protected $emailValidator;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new EmailExampleGetFormPage.
   *
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Component\Utility\EmailValidator $email_validator
   *   The email validator.
   */
  public function __construct(MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager, EmailValidator $email_validator) {
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
    $this->emailValidator = $email_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $form = new static(
      $container->get('plugin.manager.mail'),
      $container->get('language_manager'),
      $container->get('email.validator')
    );
    $form->setMessenger($container->get('messenger'));
    $form->setStringTranslation($container->get('string_translation'));
    return $form;
  }


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'page_notifications_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_page_info = \Drupal::service('load.databaseinnfo.service')->getCurrentPageInfo();
    $preload_template = \Drupal::service('load.databaseinnfo.service')->get_notify_email_template();
    //TODO use  $current_page_info["current_node"] or $current_page_info["current_path"], to reload the page
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $current_uri_row = \Drupal::request()->getRequestUri();
    $current_uri_pieces = explode("?", $current_uri_row);
    $current_uri = $current_uri_pieces[0];
    $reload_page_link = $host . $current_uri;

    $form['#attached']['library'][] = 'page_notifications/recaptcha';
    $form['#prefix'] = '<div id="page-notifications-block-container">';
    $form['#suffix'] = '</div>';

    $route_match = \Drupal::routeMatch();
    if ($route_match->getRouteName() == 'entity.node.canonical') {
      $node = \Drupal::routeMatch()->getParameter('node');
        if ($node instanceof \Drupal\node\NodeInterface) {
          $node = \Drupal\node\Entity\Node::load($node->id());
          $node_title = $node->getTitle();
          $node_node_url = $node->toUrl()->setAbsolute()->toString();

          $block_header_replacements = array(
            '[notify_user_name]' => '',
            '[notify_user_email]' => '',
            '[notify_verify_url]' => '',
            '[notify_subscribe_url]' => '',
            '[notify_unsubscribe_url]' => '',
            '[notify_user_subscribtions]' => '',
            '[notify_node_title]' => $node_title,
            '[notify_node_url]' => $node_node_url,
            '[notify_notes]' => '',
          );

          $tokanized_notify_body = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($preload_template['body'], $block_header_replacements);
          $form['block_header'] = [
            '#type' => 'processed_text',
            '#text' => $this->t($tokanized_notify_body),
            "#processed" => true,
            '#format' => 'full_html',
          ];
          $form['container'] = [
            '#type' => 'container',
            '#attributes' => ['id' => 'page-notifications-block-container'],
          ];
          $form['container']['box'] = [];
          $form['current_node'] = [
            '#type' => 'hidden',
            '#value' => $current_page_info["current_node"],
          ];
          $form['current_path'] = [
            '#type' => 'hidden',
            '#value' => $current_page_info["current_path"],
          ];
          $form['email_notify'] = [
            '#type' => 'email',
            '#title' => $this->t('Enter your E-mail Address:'),
            '#required' => true,
          ];
          $form['actions'] = [
            '#type' => 'actions',
          ];
          $moduleHandler = \Drupal::service('module_handler');
          if ($moduleHandler->moduleExists('recaptcha')) {
            $form['recaptcha'] = [
              '#markup' => pageNotificationsRecaptchaHtml(),
            ];
          }
          $form['submit'] = [
            '#type' => 'submit',
            '#ajax' => [
              'callback' => '::promptCallback',
              'wrapper' => 'page-notifications-block-container',
            ],
            '#value' => $this->t('Submit'),
          ];
          $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
          $form['#cache'] = ['max-age' => 0];
          return $form;
        }
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$this->emailValidator->isValid($form_state->getValue('email_notify'))) {
      $form_state->setErrorByName('email', $this->t('That e-mail address is not valid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * Callback for submit_driven example.
   *
   * Select the 'box' element, change the markup in it, and return it as a
   * renderable array.
   *
   * @return array
   *   Renderable array (the box element)
   */

  public function promptCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $element = $form['container'];

    if (!pageNotificationsRecaptchaVerify()) {
      $template_info = \Drupal::service('load.databaseinnfo.service')->get_notify_email_template();
      $element['box'] = [
        '#type' => 'processed_text',
        '#text' => $this->t($template_info['error_web_page_message']),
        '#format' => 'full_html',
      ];
      return $element;
   }
    if ($form_state->getValue('email_notify') && $form_state->getValue('current_node')) { //current_node
      $email_notify = $form_state->getValue('email_notify');
      $node_id = $form_state->getValue('current_node');
      $template_info = \Drupal::service('load.databaseinnfo.service')->get_notify_email_template();
      $record_exist = \Drupal::service('load.databaseinnfo.service')->checkIfRecordExistNode($email_notify, $node_id);

      if ($record_exist) {
        $unsubscribe_link = $host . "/page-notifications/unsubscribe/" . $record_exist['field_page_notify_node_id'] . '-' . $record_exist['field_page_notify_token'];
        $node = \Drupal\node\Entity\Node::load($node_id);
        $node_title = $node->getTitle();
        $subscribed_node_url = $node->toUrl()->setAbsolute()->toString();

        $replacements = array(
          '[notify_user_name]' => '',
          '[notify_user_email]' => $email_notify,
          '[notify_verify_url]' => '',
          '[notify_subscribe_url]' => '',
          '[notify_unsubscribe_url]' => $unsubscribe_link,
          '[notify_user_subscribtions]' => '',
          '[notify_node_title]' => $node_title,
          '[notify_node_url]' => $subscribed_node_url,
          '[notify_notes]' => '',
        );

        $tokanized_web_message = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['record_exist_verify_web_page_message'], $replacements);

        $host = \Drupal::request()->getSchemeAndHttpHost();
        $element['box'] = [
          '#type' => 'processed_text',
          '#text' => $this->t($tokanized_web_message),
          '#format' => 'full_html',
        ];
        return $element;
      }
      else {
        if ($template_info['from_email']) {
          $from = $template_info['from_email'];
        }
        else {
          $from = \Drupal::config('system.site')->get('mail');
        }
        //create reload page
        $host = \Drupal::request()->getSchemeAndHttpHost();
        $current_uri_row = \Drupal::request()->getRequestUri();
        $current_uri_pieces = explode("?", $current_uri_row);
        $current_uri = $current_uri_pieces[0];
        $reload_page_link = $host . $current_uri;

        // Create tokens
        $subscription_token = page_notifications_generateRandomString();

        $node = \Drupal\node\Entity\Node::load($node_id);
        $node_title = $node->getTitle();
        $subscribed_node_url = $node->toUrl()->setAbsolute()->toString();
        $confrm_url = $host . "/page-notifications/confirmation/" . $form_state->getValue('email_notify') . "/" . $form_state->getValue('current_node') . "-" . $subscription_token;

        $subject_replacements = array(
          '[notify_user_name]' => '',
          '[notify_user_email]' => $email_notify,
          '[notify_verify_url]' => '',
          '[notify_subscribe_url]' => '',
          '[notify_unsubscribe_url]' => '',
          '[notify_user_subscribtions]' => '',
          '[notify_node_title]' => $node_title,
          '[notify_node_url]' => '',
          '[notify_notes]' => '',
        );
        $body_replacements = array(
          '[notify_user_name]' => '',
          '[notify_user_email]' => $email_notify,
          '[notify_verify_url]' => $confrm_url,
          '[notify_subscribe_url]' => '',
          '[notify_unsubscribe_url]' => '',
          '[notify_user_subscribtions]' => '',
          '[notify_node_title]' => $node_title,
          '[notify_node_url]' => $subscribed_node_url,
          '[notify_notes]' => '',
        );

        $tokanized_subject = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['verification_email_subject'], $subject_replacements);
        $tokanized_body = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['verification_email_text'], $body_replacements);
        $tokanized_web_page_message = \Drupal::service('load.databaseinnfo.service')->page_notifications_process_tokens($template_info['sent_verify_web_page_message'], $body_replacements);

        $element['box'] = [
          '#type' => 'processed_text',
          '#text' => $this->t($tokanized_web_page_message),
          '#format' => 'full_html',
        ];

        $message['to'] = $email_notify;
        $message['subject'] = $tokanized_subject;
        $message['body'] = $tokanized_body;
        $result = \Drupal::service('plugin.manager.mail')->mail(
           'page_notifications',
           'verification_email',
           $email_notify,
           \Drupal::languageManager()->getDefaultLanguage()->getId(),
           $message
         );

         if ($result['result'] !== true) {
           //$email_error_message = 'There was a problem sending your message, it was NOT sent, please contact us by <a href = "mailto:' . $from . ' ">Email</a>';

           $response->addCommand(new HtmlCommand('.page-notifications-form', $element));
         }
         else {
           $response->addCommand(new HtmlCommand('.page-notifications-form', $element));
         }
         return $response;
      }
    }
  }
}


function pageNotificationsRecaptchaHtml() {
  $site_key = \Drupal::config('recaptcha.settings')->get('site_key');
  if (!is_null($site_key)) {
    define('RECAPTCHA_SITEKEY', $site_key);
    return '<div class="g-recaptcha mb-3" data-sitekey="' . RECAPTCHA_SITEKEY . '"></div>';
  }
}

function pageNotificationsPost(string $url, array $postdata) {
  $content = http_build_query($postdata);
  $opts = array('http' =>
    array(
      'method' => 'POST',
      'header' => 'Content-Type: application/x-www-form-urlencoded',
      'content' => $content
    )
  );
  $context = stream_context_create($opts);
  $result = file_get_contents($url, false, $context);
  return $result;
}

function pageNotificationsRecaptchaVerify() {
  if (!isset($_POST['g-recaptcha-response'])) {
    return false;
  }
  $response = filter_input(INPUT_POST, 'g-recaptcha-response', FILTER_SANITIZE_STRING);
  $secret_key = \Drupal::config('recaptcha.settings')->get('secret_key');
  define('RECAPTCHA_SECRETKEY', $secret_key);
  define('RECAPTCHA_URL', 'https://www.google.com/recaptcha/api/siteverify');
  $json = pageNotificationsPost(RECAPTCHA_URL, ['secret' => RECAPTCHA_SECRETKEY, 'response' => $response]);
  $data = json_decode($json, true);
  if ($data['success'] == 1) {
    return true;
  }
  return false;
}



function page_notifications_generateRandomString($length = 10) {
  $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
  $charactersLength = strlen($characters);
  $randomString = '';
  for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
  }
  return $randomString;
}
